@@ -138,6 +138,79 @@ make_roam_request(struct wd_request_context *context, struct roam_req_info *roam
     free(uri);
 }
 
+static void
+process_auth_server_login2(struct evhttp_request *req, void *ctx)
+{
+	auth_req_info *auth = ((struct wd_request_context *)ctx)->data;
+	debug(LOG_DEBUG, "process auth server login2 response");
+	
+    char buffer[MAX_BUF] = {0};
+    if (evbuffer_remove(evhttp_request_get_input_buffer(req), buffer, MAX_BUF-1) > 0 ) {
+	} else {
+		free(auth);
+		return;
+	}
+	
+	json_object *json_ret = json_tokener_parse(buffer);
+	json_object *ret_code = NULL;
+	json_object_object_get_ex(json_ret, "ret_code", &ret_code);
+	int retCode = json_object_get_int(ret_code);
+	if (retCode != 0) {
+		free(auth);
+		debug(LOG_INFO, "add test client failure: %d", retCode);
+		return;
+	}
+	
+	json_object *client = NULL;
+	if( json_object_object_get_ex(json_ret, "client", &client)) {
+		add_online_client(auth->ip, auth->mac, client);
+	} else {
+		debug(LOG_ERR, "no roam client info!!!!!");
+	}
+	
+	json_object_put(json_ret);
+	free(auth);
+}
+
+/**
+ * @brief get login2 request uri
+ * 
+ */ 
+static char *
+get_login2_request_uri(s_config *config, t_auth_serv *auth_server, const auth_req_info *auth)
+{
+    char *login2_uri = NULL;
+    safe_asprintf(&login2_uri, "%slogin2?gw_id=%s&gw_address=%s&gw_port=%d&mac=%s&channel_path=%s&cltIp=%s", 
+        auth_server->authserv_path,
+        config->gw_id,
+		config->gw_address,
+		config->gw_port,
+		auth->mac,
+		g_channel_path?g_channel_path:"null",
+		auth->ip);
+    return login2_uri;
+}
+
+/**
+ * @brief wifidog make auth quest to auth server
+ * 
+ * @param auth The auth request data, need to be free 
+ * 
+ */ 
+void 
+make_auth_request(struct wd_request_context *context, auth_req_info *auth)
+{
+	char *uri = get_login2_request_uri(config_get_config(), get_auth_server(), auth);
+    debug(LOG_DEBUG, "login2 request uri [%s]", uri);
+
+    struct evhttp_connection *evcon = NULL;
+    struct evhttp_request *req      = NULL;
+    context->data = auth; 
+    wd_make_request(context, &evcon, &req, process_auth_server_login2);
+    evhttp_make_request(evcon, req, EVHTTP_REQ_GET, uri);
+    free(uri);
+}
+
 /**
  * @brief get auth counter v2 uri
  * @return need to be free by the caller
@@ -554,6 +627,15 @@ process_auth_server_counter_v2(struct evhttp_request *req, void *ctx)
         authresponse.authcode = auth_code;
         client_counter_request_reply_v2(&authresponse, req, ctx);
     }
+	
+	json_object *j_client_timeout = json_object_object_get(j_result, "clientTimeout");
+	if (j_client_timeout) {
+		int clientTimeout = json_object_get_int(j_client_timeout);
+		s_config *config = config_get_config();
+		LOCK_CLIENT_LIST();
+		config->clienttimeout = clientTimeout;
+		UNLOCK_CLIENT_LIST();
+	}
     
 ERR:
     if (j_result) json_object_put(j_result);
