@@ -41,7 +41,7 @@
 
 
 //>>> liudf added 20160114
-const char	*g_inner_trusted_domains = "www.kunteng.org.cn,wificoin.club,wfc.wificoin.club,wfc.xyblock.net,talkblock.org,mqtt.kunteng.org.cn,log1.kunteng.org.cn";
+const char	*g_inner_trusted_domains = "";
 
 /** @internal
  * Holds the current configuration of the gateway */
@@ -84,6 +84,7 @@ typedef enum {
 	oAuthServMsgScriptPathFragment,
 	oAuthServPingScriptPathFragment,
 	oAuthServAuthScriptPathFragment,
+	oAuthServWsScriptPathFragment,
 	oHTTPDMaxConn,
 	oHTTPDName,
 	oHTTPDRealm,
@@ -119,7 +120,10 @@ typedef enum {
 	oMQTT,
 	oMQTTServer,
 	oMQTTServerPort,
+	oMQTTUsername,
+	oMQTTPassword,
 	oDNSTimeout,
+	oFW4Enable,
 } OpCodes;
 
 /** @internal
@@ -153,6 +157,7 @@ static const struct {
 	"msgscriptpathfragment", oAuthServMsgScriptPathFragment}, {
 	"pingscriptpathfragment", oAuthServPingScriptPathFragment}, {
 	"authscriptpathfragment", oAuthServAuthScriptPathFragment}, {
+	"wsscriptpathfragment", oAuthServWsScriptPathFragment}, {
 	"firewallruleset", oFirewallRuleSet}, {
 	"firewallrule", oFirewallRule}, {
 	"trustedmaclist", oTrustedMACList}, {
@@ -179,7 +184,10 @@ static const struct {
 	"mqtt", oMQTT}, {
 	"serveraddr", oMQTTServer}, {
 	"serverport", oMQTTServerPort}, {
+	"mqttUsername", oMQTTUsername}, {
+	"mqttPassword", oMQTTPassword}, {
 	"dnstimeout",oDNSTimeout},{
+	"fw4enable",oFW4Enable},{
     NULL, oBadOption},};
 
 static void config_notnull(const void *, const char *);
@@ -269,13 +277,16 @@ config_init(void)
 
 	t_mqtt_server *mqtt_server = (t_mqtt_server *)malloc(sizeof(t_mqtt_server));
 	memset(mqtt_server, 0, sizeof(t_mqtt_server));
-	mqtt_server->hostname   = safe_strdup(DEFAULT_MQTT_SERVER);
 	mqtt_server->port	   = 8883;
 	mqtt_server->cafile	 = safe_strdup(DEFAULT_CA_CRT_FILE);
 	mqtt_server->crtfile	= NULL;
 	mqtt_server->keyfile	= NULL;
+	mqtt_server->username	= NULL;
+	mqtt_server->password	= NULL;
 	config.mqtt_server  = mqtt_server;
 
+	config.fw4_enable = 1;
+
 	debugconf.log_stderr = 1;
 	debugconf.debuglevel = DEFAULT_DEBUGLEVEL;
 	debugconf.syslog_facility = DEFAULT_SYSLOG_FACILITY;
@@ -316,23 +327,28 @@ Parses auth server information
 static void
 parse_auth_server(FILE * file, const char *filename, int *linenum)
 {
-	char *host = NULL,
-		*path = NULL,
-		*loginscriptpathfragment = NULL,
-		*portalscriptpathfragment = NULL,
-		*msgscriptpathfragment = NULL,
-		*pingscriptpathfragment = NULL, *authscriptpathfragment = NULL, line[MAX_BUF], *p1, *p2;
+	char 	*host = NULL;
+	char	*path = NULL;
+	char	*loginscriptpathfragment = NULL;
+	char	*portalscriptpathfragment = NULL;
+	char	*msgscriptpathfragment = NULL;
+	char	*pingscriptpathfragment = NULL;
+	char	*authscriptpathfragment = NULL;
+	char	*authwsscriptpathfragment = NULL;
+	char line[MAX_BUF], *p1, *p2;
 	int http_port, ssl_port, ssl_available, opcode;
 	int connect_timeout;
 	t_auth_serv *new, *tmp;
 
 	/* Defaults */
 	path = safe_strdup(DEFAULT_AUTHSERVPATH);
-	loginscriptpathfragment = safe_strdup(DEFAULT_AUTHSERVLOGINPATHFRAGMENT);
-	portalscriptpathfragment = safe_strdup(DEFAULT_AUTHSERVPORTALPATHFRAGMENT);
-	msgscriptpathfragment = safe_strdup(DEFAULT_AUTHSERVMSGPATHFRAGMENT);
-	pingscriptpathfragment = safe_strdup(DEFAULT_AUTHSERVPINGPATHFRAGMENT);
-	authscriptpathfragment = safe_strdup(DEFAULT_AUTHSERVAUTHPATHFRAGMENT);
+	loginscriptpathfragment 	= safe_strdup(DEFAULT_AUTHSERVLOGINPATHFRAGMENT);
+	portalscriptpathfragment 	= safe_strdup(DEFAULT_AUTHSERVPORTALPATHFRAGMENT);
+	msgscriptpathfragment 		= safe_strdup(DEFAULT_AUTHSERVMSGPATHFRAGMENT);
+	pingscriptpathfragment 		= safe_strdup(DEFAULT_AUTHSERVPINGPATHFRAGMENT);
+	authscriptpathfragment 		= safe_strdup(DEFAULT_AUTHSERVAUTHPATHFRAGMENT);
+	authwsscriptpathfragment 	= safe_strdup(DEFAULT_AUTHSERVWSPATHFRAGMENT);
+	
 	http_port = DEFAULT_AUTHSERVPORT;
 	ssl_port = DEFAULT_AUTHSERVSSLPORT;
 	ssl_available = DEFAULT_AUTHSERVSSLAVAILABLE;
@@ -411,6 +427,10 @@ parse_auth_server(FILE * file, const char *filename, int *linenum)
 				free(authscriptpathfragment);
 				authscriptpathfragment = safe_strdup(p2);
 				break;
+			case oAuthServWsScriptPathFragment:
+				free(authwsscriptpathfragment);
+				authwsscriptpathfragment = safe_strdup(p2);
+				break;
 			case oAuthServSSLPort:
 				ssl_port = atoi(p2);
 				break;
@@ -445,6 +465,7 @@ parse_auth_server(FILE * file, const char *filename, int *linenum)
 		free(msgscriptpathfragment);
 		free(portalscriptpathfragment);
 		free(loginscriptpathfragment);
+		free(authwsscriptpathfragment);
 		return;
 	}
 
@@ -463,6 +484,7 @@ parse_auth_server(FILE * file, const char *filename, int *linenum)
 	new->authserv_msg_script_path_fragment = msgscriptpathfragment;
 	new->authserv_ping_script_path_fragment = pingscriptpathfragment;
 	new->authserv_auth_script_path_fragment = authscriptpathfragment;
+	new->authserv_ws_script_path_fragment	= authwsscriptpathfragment;
 	new->authserv_http_port = http_port;
 	new->authserv_ssl_port 	= ssl_port;
 	new->authserv_fd		= -1;
@@ -486,7 +508,7 @@ parse_auth_server(FILE * file, const char *filename, int *linenum)
 static void
 parse_mqtt_server(FILE * file, const char *filename, int *linenum)
 {
-	char *host = NULL, line[MAX_BUF], *p1, *p2;
+	char *host = NULL, *username = NULL, *password = NULL, line[MAX_BUF], *p1, *p2;
 	int port = 0, opcode;
 	t_mqtt_server *mqtt_server = config.mqtt_server;
 
@@ -533,6 +555,12 @@ parse_mqtt_server(FILE * file, const char *filename, int *linenum)
 			case oMQTTServer:
 				host = safe_strdup(p2);
 				break;
+			case oMQTTUsername:
+				username = safe_strdup(p2);
+				break;
+			case oMQTTPassword:
+				password = safe_strdup(p2);
+				break;
 			case oMQTTServerPort:
 				port = atoi(p2);
 				break;
@@ -556,6 +584,8 @@ parse_mqtt_server(FILE * file, const char *filename, int *linenum)
 	free(mqtt_server->hostname);
 	mqtt_server->hostname = host;
 	mqtt_server->port = port;
+	mqtt_server->username = username;
+	mqtt_server->password = password;
 
 	debug(LOG_DEBUG, "MQTT server added");
 }
@@ -967,6 +997,9 @@ config_read()
 				case oAppleCNA:
 					sscanf(p1, "%hu", &config.bypass_apple_cna);
 					break;
+				case oFW4Enable:
+					config.fw4_enable = parse_boolean_value(p1);
+					break;
 				case oBadOption:
 					/* FALL THROUGH */
 				default:
@@ -2291,8 +2324,6 @@ static void
 validate_popular_servers(void)
 {
 	if (config.popular_servers == NULL) {
-		add_popular_server("www.qq.com");
-		add_popular_server("www.kunteng.org");
 		add_popular_server("www.baidu.com");
 	}
 }
