@@ -31,6 +31,7 @@
 
 #include <arpa/inet.h>
 
+
 #include "common.h"
 #include "safe.h"
 #include "conf.h"
@@ -40,10 +41,21 @@
 #include "util.h"
 #include "client_list.h"
 #include "wd_util.h"
-#include "ipset.h"
 
-#include "fw3_iptc.h"
+#ifdef AW_FW3
+	#include <libiptc/libiptc.h>
+	#include <xtables.h>
+	#include "fw3_iptc.h"
+	#include "ipset.h"
+#else
+	#include "fw4_nft.h"
+#endif
 
+/**
+Used to supress the error output of the firewall during destruction */
+static int fw_quiet = 0;
+
+#ifdef AW_FW3
 static FILE	*f_fw_init = NULL;
 static FILE	*f_fw_destroy = NULL;
 static FILE	*f_fw_allow  = NULL;
@@ -71,9 +83,7 @@ static int __iptables_fw_destroy_mention(const char *table, const char *chain, c
 #define iptables_do_command(...) \
 	iptables_do_append_command(NULL, __VA_ARGS__)
 
-/**
-Used to supress the error output of the firewall during destruction */
-static int fw_quiet = 0;
+
 
 /** @internal
  * @brief Insert $ID$ with the gateway's id in a string.
@@ -104,27 +114,12 @@ iptables_insert_gateway_id(char **input)
 	*input = buffer;
 }
 
-int
-add_mac_to_ipset(const char *name, const char *mac, int timeout)
-{
-	char *ipset_name =  NULL;
-	if(name == NULL)
-		return -1;
- 
-	ipset_name = safe_malloc(strlen(name) + 1);
-	memcpy(ipset_name, name, strlen(name));
-	iptables_insert_gateway_id(&ipset_name);
-
-	int nret = add_to_ipset(ipset_name, mac, timeout);
-	free(ipset_name);
-	return nret;
-}
-
 /** @internal
  * */
 static int
 add_ip_to_ipset(const char *name, const char *ip, int remove)
 {
+
 	char *ipset_name =  NULL;
 	if(name == NULL)
 		return -1;
@@ -354,16 +349,78 @@ iptables_load_ruleset(const char *table, const char *ruleset, const char *chain,
 
 	debug(LOG_DEBUG, "Ruleset %s loaded into table %s, chain %s", ruleset, table, chain);
 }
+#else
+/** @internal
+ * 
+*/
+static int
+nftables_do_command(const char *format, ...)
+{
+	va_list vlist;
+	char *fmt_cmd;
+	char *cmd;
+	int rc;
+
+	va_start(vlist, format);
+	safe_vasprintf(&fmt_cmd, format, vlist);
+	va_end(vlist);
+
+	safe_asprintf(&cmd, "nft %s", fmt_cmd);
+	free(fmt_cmd);
+
+	debug(LOG_DEBUG, "Executing command: %s", cmd);
+
+	rc = execute(cmd, fw_quiet);
+
+	if (rc != 0) {
+		// If quiet, do not display the error
+		if (fw_quiet == 0)
+			debug(LOG_ERR, "nft command failed(%d): %s", rc, cmd);
+		else if (fw_quiet == 1)
+			debug(LOG_DEBUG, "nft command failed(%d): %s", rc, cmd);
+	}
+
+	free(cmd);
+
+	return rc;
+}
+#endif
+
+int
+add_mac_to_ipset(const char *name, const char *mac, int timeout)
+{
+#ifdef AW_FW3
+	char *ipset_name =  NULL;
+	if(name == NULL)
+		return -1;
+ 
+	ipset_name = safe_malloc(strlen(name) + 1);
+	memcpy(ipset_name, name, strlen(name));
+	iptables_insert_gateway_id(&ipset_name);
+
+	int nret = add_to_ipset(ipset_name, mac, timeout);
+	free(ipset_name);
+	return nret;
+#else
+	return 0;
+#endif
+}
 
 void
 iptables_fw_clear_authservers(void)
 {
+#ifdef AW_FW3
 	iptables_do_command("-t filter -F " CHAIN_AUTHSERVERS);
 	iptables_do_command("-t nat -F " CHAIN_AUTHSERVERS);
+#else
+#endif
 }
 
-void
-iptables_fw_set_authservers(void *handle)
+
+#ifdef AW_FW3
+
+static void
+fw3_set_authservers(void *handle)
 {
 	const s_config *config;
 	t_auth_serv *auth_server;
@@ -383,22 +440,59 @@ iptables_fw_set_authservers(void *handle)
 	}
 }
 
+#else
+
+static void
+fw4_set_authservers(void *handle)
+{
+	// add auth_server->last_ip to nftables set set_wifidogx_auth_servers
+	const s_config *config;
+	t_auth_serv *auth_server;
+
+	config = config_get_config();
+	for (auth_server = config->auth_servers; auth_server != NULL; auth_server = auth_server->next) {
+		if (auth_server->last_ip && strcmp(auth_server->last_ip, "0.0.0.0") != 0) {
+			debug(LOG_DEBUG, "the last ip: %s", auth_server->last_ip);
+			nftables_do_command("add element inet fw4 set_wifidogx_auth_servers { %s }", auth_server->last_ip);
+		}
+	}
+}
+
+#endif
+
+void
+iptables_fw_set_authservers(void *handle)
+{
+#ifdef AW_FW3
+	fw3_set_authservers(handle);
+#else
+	fw4_set_authservers(handle);
+#endif
+}
+
 void
 iptables_fw_refresh_user_domains_trusted(void)
 {
+#ifdef AW_FW3
 	iptables_fw_clear_user_domains_trusted();
 	iptables_fw_set_user_domains_trusted();
+#else
+#endif
 }
 
 void
 iptables_fw_clear_user_domains_trusted(void)
 {
+#ifdef AW_FW3
 	iptables_flush_ipset(CHAIN_DOMAIN_TRUSTED);
+#else
+#endif
 }
 
 void
 iptables_fw_set_user_domains_trusted(void)
 {
+#ifdef AW_FW3
 	const s_config *config;
 	t_domain_trusted *domain_trusted = NULL;
 
@@ -414,22 +508,28 @@ iptables_fw_set_user_domains_trusted(void)
 	}
 
 	UNLOCK_DOMAIN();
+#else
+#endif
 }
 
 // set inner trusted domains
 void
 iptables_fw_clear_ipset_domains_trusted(void)
 {
+#ifdef AW_FW3
 	char f_ipset_name[128] = {0};
 	snprintf(f_ipset_name, 128, "%s/%s", DNSMASQ_CONF_D, CHAIN_IPSET_TDOMAIN);
 
 	remove(f_ipset_name);
 	iptables_flush_ipset(CHAIN_IPSET_TDOMAIN);
+#else
+#endif
 }
 
 void
 iptables_fw_set_ipset_domains_trusted(void)
 {
+#ifdef AW_FW3
 	const s_config *config;
 	t_domain_trusted *domain_trusted = NULL;
 	FILE *fd_ipset = NULL;
@@ -460,19 +560,27 @@ iptables_fw_set_ipset_domains_trusted(void)
 		remove(f_ipset_name);
 
 	iptables_flush_ipset(CHAIN_IPSET_TDOMAIN);
+#else
+#endif
 }
 
 void
 iptables_fw_refresh_inner_domains_trusted(void)
 {
+#ifdef AW_FW3
 	iptables_fw_clear_inner_domains_trusted();
 	iptables_fw_set_inner_domains_trusted();
+#else
+#endif
 }
 
 void
 iptables_fw_clear_inner_domains_trusted(void)
 {
+#ifdef AW_FW3
 	iptables_flush_ipset(CHAIN_INNER_DOMAIN_TRUSTED);
+#else
+#endif
 }
 
 /**
@@ -482,6 +590,7 @@ iptables_fw_clear_inner_domains_trusted(void)
 void
 iptables_fw_set_inner_domains_trusted(void)
 {
+#ifdef AW_FW3
 	const s_config *config;
 	t_domain_trusted *domain_trusted = NULL;
 
@@ -497,30 +606,42 @@ iptables_fw_set_inner_domains_trusted(void)
 	}
 
 	UNLOCK_DOMAIN();
+#else
+#endif
 }
 
 
 void
 iptables_fw_clear_roam_maclist(void)
 {
+#ifdef AW_FW3
 	iptables_flush_ipset(CHAIN_ROAM);
+#else
+#endif
 }
 
 void
 iptables_fw_set_roam_mac(const char *mac)
 {
+#ifdef AW_FW3
 	ipset_do_command("add " CHAIN_ROAM " %s", mac);
+#else
+#endif
 }
 
 void
 iptables_fw_clear_trusted_maclist(void)
 {
+#ifdef AW_FW3
 	iptables_flush_ipset(CHAIN_TRUSTED);
+#else
+#endif
 }
 
 void
 iptables_fw_set_trusted_maclist(void)
 {
+#ifdef AW_FW3
 	const s_config *config;
 	t_trusted_mac *p = NULL;
 
@@ -530,23 +651,32 @@ iptables_fw_set_trusted_maclist(void)
 	for (p = config->trustedmaclist; p != NULL; p = p->next)
 		ipset_do_command("add " CHAIN_TRUSTED " %s", p->mac);
 	UNLOCK_CONFIG();
+#else
+#endif
 }
 
 void
 iptables_fw_set_trusted_mac(const char *mac)
 {
+#ifdef AW_FW3
 	ipset_do_command("add " CHAIN_TRUSTED " %s", mac);
+#else
+#endif
 }
 
 void
 iptables_fw_clear_trusted_local_maclist(void)
 {
+#ifdef AW_FW3
 	iptables_flush_ipset(CHAIN_TRUSTED_LOCAL);
+#else
+#endif
 }
 
 void
 iptables_fw_set_trusted_local_maclist(void)
 {
+#ifdef AW_FW3
 	const s_config *config;
 	t_trusted_mac *p = NULL;
 
@@ -556,17 +686,23 @@ iptables_fw_set_trusted_local_maclist(void)
 	for (p = config->trusted_local_maclist; p != NULL; p = p->next)
 		ipset_do_command("add " CHAIN_TRUSTED_LOCAL " %s", p->mac);
 	UNLOCK_CONFIG();
+#else
+#endif
 }
 
 void
 iptables_fw_clear_untrusted_maclist(void)
 {
+#ifdef AW_FW3
 	iptables_flush_ipset(CHAIN_UNTRUSTED);
+#else
+#endif
 }
 
 void
 iptables_fw_set_untrusted_maclist(void)
 {
+#ifdef AW_FW3
 	const s_config *config;
 	t_trusted_mac *p = NULL;
 
@@ -577,18 +713,24 @@ iptables_fw_set_untrusted_maclist(void)
 		ipset_do_command("add " CHAIN_UNTRUSTED " %s", p->mac);
 		//add_mac_to_ipset(CHAIN_UNTRUSTED, p->mac, 0);
 	UNLOCK_CONFIG();
+#else
+#endif
 }
 
 void
 iptables_fw_set_mac_temporary(const char *mac, int which)
 {
+#ifdef AW_FW3
 	if(which == 0) { // trusted
 		ipset_do_command("add " CHAIN_TRUSTED " %s timeout 60 ", mac);
 	} else if(which == 1) { // untrusted
 		ipset_do_command("add " CHAIN_UNTRUSTED " %s timeout 60 ", mac);
 	}
+#else
+#endif
 }
 
+#ifdef AW_FW3
 static void
 f_fw_init_close()
 {
@@ -670,10 +812,12 @@ f_fw_allow_open()
 		fprintf(f_fw_allow, "#!/bin/sh\n");
 	}
 }
+#endif
 
 void
 iptables_fw_save_online_clients()
 {
+#ifdef AW_FW3
 	t_client *sublist, *current;
 
 	LOCK_CLIENT_LIST();
@@ -695,19 +839,20 @@ iptables_fw_save_online_clients()
 	}
 
 	f_fw_allow_close();
+#else
+#endif
 }
 
-/** Initialize the firewall rules
-*/
-int
-iptables_fw_init(void)
+#ifdef AW_FW3
+
+static int 
+fw3_int(void)
 {
 	const s_config *config;
 	char *ext_interface = NULL;
 	int gw_port = 0;
 	int gw_https_port = 0;
 	int gw_http_port = 0;
-	int proxy_port;
 	fw_quiet = 0;
 	int got_authdown_ruleset = NULL == get_ruleset(FWRULESET_AUTH_IS_DOWN) ? 0 : 1;
 
@@ -738,6 +883,8 @@ iptables_fw_init(void)
 		return 0;
 	}
 
+	ipset_init();
+
 	// add ipset support
 	ipset_do_command("create " CHAIN_TRUSTED " hash:mac timeout 0 ");
 	ipset_do_command("create " CHAIN_TRUSTED_LOCAL " hash:mac timeout 0 ");
@@ -799,40 +946,21 @@ iptables_fw_init(void)
 	/* Create new chains */
 	iptables_do_append_command(handle, "-t nat -N " CHAIN_UNTRUSTED);
 	iptables_do_append_command(handle, "-t nat -N " CHAIN_OUTGOING);
-	iptables_do_append_command(handle, "-t nat -N " CHAIN_TO_ROUTER);
 	iptables_do_append_command(handle, "-t nat -N " CHAIN_TO_INTERNET);
 	iptables_do_append_command(handle, "-t nat -N " CHAIN_GLOBAL);
 	iptables_do_append_command(handle, "-t nat -N " CHAIN_UNKNOWN);
 	iptables_do_append_command(handle, "-t nat -N " CHAIN_AUTHSERVERS);
 	iptables_do_append_command(handle, "-t nat -N " CHAIN_DOMAIN_TRUSTED);
-	iptables_do_append_command(handle, "-t nat -N " CHAIN_TO_PASS);
 	if (got_authdown_ruleset)
 		iptables_do_append_command(handle, "-t nat -N " CHAIN_AUTH_IS_DOWN);
 
-	// add this rule for mac blacklist
-	iptables_do_append_command(handle, "-t nat -A PREROUTING -i %s -j " CHAIN_UNTRUSTED, config->gw_interface);
-	iptables_do_append_command(handle, "-t nat -A " CHAIN_UNTRUSTED " -p tcp -m set --match-set " CHAIN_UNTRUSTED " src -j REDIRECT --to-ports 44444");
-	iptables_do_append_command(handle, "-t nat -A " CHAIN_UNTRUSTED " -p udp -m set --match-set " CHAIN_UNTRUSTED " src -j REDIRECT --to-ports 44444");
-
 	/* Assign links and rules to these new chains */
 	iptables_do_append_command(handle, "-t nat -A PREROUTING -i %s -j " CHAIN_OUTGOING, config->gw_interface);
 
-	iptables_do_append_command(handle, "-t nat -A " CHAIN_OUTGOING " -d %s -j " CHAIN_TO_ROUTER, config->gw_address);
-	iptables_do_append_command(handle, "-t nat -A " CHAIN_TO_ROUTER " -j ACCEPT");
+	iptables_do_append_command(handle, "-t nat -A " CHAIN_OUTGOING " -d %s -j ACCEPT" , config->gw_address);
 
 	iptables_do_append_command(handle, "-t nat -A " CHAIN_OUTGOING " -j " CHAIN_TO_INTERNET);
-	iptables_do_append_command(handle, "-t nat -A " CHAIN_TO_INTERNET " -j " CHAIN_TO_PASS);
 
-	if ((proxy_port = config_get_config()->proxy_port) != 0) {
-		debug(LOG_DEBUG, "Proxy port set, setting proxy rule");
-
-		iptables_do_append_command(handle, "-t nat -A " CHAIN_TO_PASS
-							" -p tcp --dport 80 -m mark --mark 0x%x0000/0xff0000 -j REDIRECT --to-port %u", FW_MARK_KNOWN,
-							proxy_port);
-		iptables_do_append_command(handle, "-t nat -A " CHAIN_TO_PASS
-							" -p tcp --dport 80 -m mark --mark 0x%x0000/0xff0000 -j REDIRECT --to-port %u", FW_MARK_PROBATION,
-							proxy_port);
-	}
 
 	iptables_do_append_command(handle, "-t nat -A " CHAIN_TO_INTERNET " -m mark --mark 0x%x0000/0xff0000 -j ACCEPT", FW_MARK_KNOWN);
 	iptables_do_append_command(handle, "-t nat -A " CHAIN_TO_INTERNET " -m mark --mark 0x%x0000/0xff0000 -j ACCEPT", FW_MARK_PROBATION);
@@ -847,6 +975,11 @@ iptables_fw_init(void)
 	iptables_do_append_command(handle, "-t nat -A " CHAIN_DOMAIN_TRUSTED " -m set --match-set " CHAIN_INNER_DOMAIN_TRUSTED " dst -j ACCEPT");
 	iptables_do_append_command(handle, "-t nat -A " CHAIN_UNKNOWN " -j " CHAIN_GLOBAL);
 	iptables_load_ruleset("nat", FWRULESET_GLOBAL, CHAIN_GLOBAL, handle);
+	
+	// add this rule for mac blacklist
+	iptables_do_append_command(handle, "-t nat -A " CHAIN_UNKNOWN " -j " CHAIN_UNTRUSTED);
+	iptables_do_append_command(handle, "-t nat -A " CHAIN_UNTRUSTED " -m set --match-set " CHAIN_UNTRUSTED " src -j REJECT --reject-with icmp-port-unreachable");
+
 
 	if (got_authdown_ruleset) {
 		iptables_do_append_command(handle, "-t nat -A " CHAIN_UNKNOWN " -j " CHAIN_AUTH_IS_DOWN);
@@ -876,7 +1009,6 @@ iptables_fw_init(void)
 	iptables_do_append_command(handle, "-t filter -N " CHAIN_TO_INTERNET);
 	iptables_do_append_command(handle, "-t filter -N " CHAIN_AUTHSERVERS);
 	iptables_do_append_command(handle, "-t filter -N " CHAIN_DOMAIN_TRUSTED);
-	iptables_do_append_command(handle, "-t filter -N " CHAIN_LOCKED);
 	iptables_do_append_command(handle, "-t filter -N " CHAIN_GLOBAL);
 	iptables_do_append_command(handle, "-t filter -N " CHAIN_VALIDATE);
 	iptables_do_append_command(handle, "-t filter -N " CHAIN_KNOWN);
@@ -887,14 +1019,15 @@ iptables_fw_init(void)
 	/* Insert at the beginning */
 	iptables_do_append_command(handle, "-t filter -I FORWARD -i %s -j " CHAIN_TO_INTERNET, config->gw_interface);
 
+#if	0
+	// when apfree wifidog work in bypass mode, need to comment this line
 	iptables_do_append_command(handle, "-t filter -A " CHAIN_TO_INTERNET " -m conntrack --ctstate INVALID -j DROP");
-
+#endif
+	
 	/* TCPMSS rule for PPPoE */
 	iptables_do_append_command(handle, "-t filter -A " CHAIN_TO_INTERNET
 						" -o %s -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu", ext_interface);
 
-	iptables_do_append_command(handle, "-t filter -A " CHAIN_TO_INTERNET " -m mark --mark 0x%x0000/0xff0000 -j " CHAIN_LOCKED, FW_MARK_LOCKED);
-	iptables_load_ruleset("filter", FWRULESET_LOCKED_USERS, CHAIN_LOCKED, handle);
 
 	iptables_do_append_command(handle, "-t filter -A " CHAIN_TO_INTERNET " -j " CHAIN_AUTHSERVERS);
 	iptables_fw_set_authservers(handle);
@@ -934,12 +1067,41 @@ iptables_fw_init(void)
 	return 1;
 }
 
-/** Remove the firewall rules
- * This is used when we do a clean shutdown of WiFiDog and when it starts to make
- * sure there are no rules left over
- */
+#else
+
+static int
+fw4_init(void)
+{
+	const s_config *config = config_get_config();
+	char *gateway_ip = NULL;
+	char *gw_interface = NULL;
+	
+	gateway_ip = config->gw_address;
+	gw_interface = config->gw_interface;
+
+	// execute nft_init
+	return nft_init(gateway_ip, gw_interface);
+}
+
+#endif
+
+/** Initialize the firewall rules
+*/
 int
-iptables_fw_destroy(void)
+iptables_fw_init(void)
+{
+	// if define AW_FW3, then fw3_init
+#ifdef AW_FW3
+	return fw3_int();
+#else
+	return fw4_init();
+#endif
+}
+
+
+#ifdef AW_FW3
+static int
+fw3_destroy(void)
 {
 	int got_authdown_ruleset = NULL == get_ruleset(FWRULESET_AUTH_IS_DOWN) ? 0 : 1;
 	fw_quiet = 1;
@@ -951,6 +1113,8 @@ iptables_fw_destroy(void)
 	} else {
 	}
 	
+	ipset_init();
+
 	/*
 	 *
 	 * Everything in the MANGLE table
@@ -988,24 +1152,20 @@ iptables_fw_destroy(void)
 	iptables_fw_destroy_mention("nat", "PREROUTING", CHAIN_UNTRUSTED, NULL);
 	iptables_fw_destroy_mention("nat", "PREROUTING", CHAIN_OUTGOING, NULL);
 	iptables_do_command("-t nat -F " CHAIN_AUTHSERVERS);
-	iptables_do_command("-t nat -F " CHAIN_TO_PASS);
 	iptables_do_command("-t nat -F " CHAIN_UNTRUSTED);
 	iptables_do_command("-t nat -F " CHAIN_DOMAIN_TRUSTED);
 	iptables_do_command("-t nat -F " CHAIN_OUTGOING);
 	if (got_authdown_ruleset)
 		iptables_do_command("-t nat -F " CHAIN_AUTH_IS_DOWN);
-	iptables_do_command("-t nat -F " CHAIN_TO_ROUTER);
 	iptables_do_command("-t nat -F " CHAIN_TO_INTERNET);
 	iptables_do_command("-t nat -F " CHAIN_GLOBAL);
 	iptables_do_command("-t nat -F " CHAIN_UNKNOWN);
 	iptables_do_command("-t nat -X " CHAIN_AUTHSERVERS);
-	iptables_do_command("-t nat -X " CHAIN_TO_PASS);
    	iptables_do_command("-t nat -X " CHAIN_UNTRUSTED);
 	iptables_do_command("-t nat -X " CHAIN_DOMAIN_TRUSTED);
 	iptables_do_command("-t nat -X " CHAIN_OUTGOING);
 	if (got_authdown_ruleset)
 		iptables_do_command("-t nat -X " CHAIN_AUTH_IS_DOWN);
-	iptables_do_command("-t nat -X " CHAIN_TO_ROUTER);
 	iptables_do_command("-t nat -X " CHAIN_TO_INTERNET);
 	iptables_do_command("-t nat -X " CHAIN_GLOBAL);
 	iptables_do_command("-t nat -X " CHAIN_UNKNOWN);
@@ -1020,7 +1180,6 @@ iptables_fw_destroy(void)
 	iptables_do_command("-t filter -F " CHAIN_TO_INTERNET);
 	iptables_do_command("-t filter -F " CHAIN_AUTHSERVERS);
 	iptables_do_command("-t filter -F " CHAIN_DOMAIN_TRUSTED);
-	iptables_do_command("-t filter -F " CHAIN_LOCKED);
 	iptables_do_command("-t filter -F " CHAIN_GLOBAL);
 	iptables_do_command("-t filter -F " CHAIN_VALIDATE);
 	iptables_do_command("-t filter -F " CHAIN_KNOWN);
@@ -1030,7 +1189,6 @@ iptables_fw_destroy(void)
 	iptables_do_command("-t filter -X " CHAIN_TO_INTERNET);
 	iptables_do_command("-t filter -X " CHAIN_AUTHSERVERS);
 	iptables_do_command("-t filter -X " CHAIN_DOMAIN_TRUSTED);
-	iptables_do_command("-t filter -X " CHAIN_LOCKED);
 	iptables_do_command("-t filter -X " CHAIN_GLOBAL);
 	iptables_do_command("-t filter -X " CHAIN_VALIDATE);
 	iptables_do_command("-t filter -X " CHAIN_KNOWN);
@@ -1054,6 +1212,33 @@ iptables_fw_destroy(void)
 	return 1;
 }
 
+#else
+
+static int
+fw4_destroy(void)
+{
+	return execute("fw4 restart", 0);
+}
+
+#endif
+
+/** Remove the firewall rules
+ * This is used when we do a clean shutdown of WiFiDog and when it starts to make
+ * sure there are no rules left over
+ */
+int 
+iptables_fw_destroy(void)
+{
+#ifdef AW_FW3
+	return fw3_destroy();
+#else
+	return fw4_destroy();
+#endif
+}
+
+
+
+#ifdef AW_FW3
 /*
  * Helper for iptables_fw_destroy
  * @param table The table to search
@@ -1116,16 +1301,23 @@ __iptables_fw_destroy_mention(const char *table, const char *chain, const char *
 
 	return (deleted);
 }
+#endif
 
 int
 iptables_fw_destroy_mention(const char *table, const char *chain, const char *mention, void *handle)
 {
+#ifdef AW_FW3
 	return __iptables_fw_destroy_mention(table, chain, mention, handle, 20);
+#else
+	return 0;
+#endif
 }
 
-/** Set if a specific client has access through the firewall */
-int
-iptables_fw_access(fw_access_t type, const char *ip, const char *mac, int tag)
+
+#ifdef AW_FW3
+
+static int
+fw3_access(fw_access_t type, const char *ip, const char *mac, int tag)
 {
 	int rc;
 
@@ -1151,8 +1343,32 @@ iptables_fw_access(fw_access_t type, const char *ip, const char *mac, int tag)
 	return rc;
 }
 
+#else
+
+static int
+fw4_access(fw_access_t type, const char *ip, const char *mac, int tag)
+{
+	return nft_fw_access(type, ip, mac, tag);
+}
+
+#endif
+
+/** Set if a specific client has access through the firewall */
 int
-iptables_fw_access_host(fw_access_t type, const char *host)
+iptables_fw_access(fw_access_t type, const char *ip, const char *mac, int tag)
+{
+#ifdef AW_FW3
+	return fw3_access(type, ip, mac, tag);
+#else
+	return fw4_access(type, ip, mac, tag);
+#endif
+}
+
+
+#ifdef AW_FW3
+
+static int 
+fw3_access_host(fw_access_t type, const char *host)
 {
 	int rc;
 
@@ -1175,10 +1391,33 @@ iptables_fw_access_host(fw_access_t type, const char *host)
 	return rc;
 }
 
-/** Set a mark when auth server is not reachable */
-int
-iptables_fw_auth_unreachable(int tag)
+#else
+
+static int 
+fw4_access_host(fw_access_t type, const char *host)
 {
+	return 0;
+}
+
+#endif
+
+int
+iptables_fw_access_host(fw_access_t type, const char *host)
+{
+#ifdef AW_FW3
+	return fw3_access_host(type, host);
+#else
+	return fw4_access_host(type, host);
+#endif
+}
+
+
+#ifdef AW_FW3
+
+static int 
+fw3_auth_unreachable(int tag)
+{
+
 	int got_authdown_ruleset = NULL == get_ruleset(FWRULESET_AUTH_IS_DOWN) ? 0 : 1;
 	if (got_authdown_ruleset)
 		return iptables_do_command("-t mangle -A " CHAIN_AUTH_IS_DOWN " -j MARK --set-mark 0x%02x0000/0xff0000", tag & 0x000000ff);
@@ -1186,10 +1425,32 @@ iptables_fw_auth_unreachable(int tag)
 		return 1;
 }
 
-/** Remove mark when auth server is reachable again */
-int
-iptables_fw_auth_reachable(void)
+#else
+
+static int
+fw4_auth_unreachable(int tag)
 {
+	return 0;
+}
+
+#endif
+
+/** Set a mark when auth server is not reachable */
+int
+iptables_fw_auth_unreachable(int tag) 
+{
+#ifdef AW_FW3
+	return fw3_auth_unreachable(tag);
+#else
+	return fw4_auth_unreachable(tag);
+#endif
+}
+
+#ifdef AW_FW3
+static int
+fw3_auth_reachable(void)
+{
+
 	int got_authdown_ruleset = NULL == get_ruleset(FWRULESET_AUTH_IS_DOWN) ? 0 : 1;
 	if (got_authdown_ruleset)
 		return iptables_do_command("-t mangle -F " CHAIN_AUTH_IS_DOWN);
@@ -1197,6 +1458,27 @@ iptables_fw_auth_reachable(void)
 		return 1;
 }
 
+#else
+
+static int
+fw4_auth_reachable(void)
+{
+	return 0;
+}
+
+#endif
+
+/** Remove mark when auth server is reachable again */
+int
+iptables_fw_auth_reachable(void)
+{
+#ifdef AW_FW3
+	return fw3_auth_reachable();
+#else	
+	return fw4_auth_reachable();
+#endif
+}
+
 /**
  * @brief get device's name from dnsmasq's dhcp.leases file 
  */
@@ -1220,9 +1502,11 @@ __get_client_name(t_client *client)
 	}
 }
 
-/** Update the counters of all the clients in the client list */
-int
-iptables_fw_counters_update(void)
+
+#ifdef AW_FW3
+
+static int
+fw3_counters_update(void)
 {
 	FILE *output;
 	char *script, ip[16] = {0}, rc;
@@ -1338,4 +1622,261 @@ iptables_fw_counters_update(void)
 	pclose(output);
 
 	return 1;
+
 }
+
+#else
+
+static int 
+fw4_counters_update()
+{
+	int nret = 0;
+
+	reset_client_list();
+
+	// get client outgoing traffic
+	char outgoing[4096] = {0};
+	nft_statistical_outgoing(outgoing, sizeof(outgoing));
+	// outgoing is json format string
+	// parse json string outgoing as jobj_outgoing
+	json_object *jobj_outgoing = json_tokener_parse(outgoing);
+	if (jobj_outgoing == NULL) {
+		debug(LOG_ERR, "fw4_counters_update(): jobj_outgoing is NULL");
+		nret = 1;
+		goto END_UPDATE;
+	}
+	// get the "nftables" json object which is an array of json objects
+	json_object *jobj_nftables = NULL;
+	if (!json_object_object_get_ex(jobj_outgoing, "nftables", &jobj_nftables)) {
+		debug(LOG_ERR, "fw4_counters_update(): jobj_nftables is NULL");
+		nret = 1;
+		goto END_UPDATE;
+	}
+	// the jobj_outgoing is an array of json objects
+	// iterate the array to get each json object
+	int i;
+	int arraylen = json_object_array_length(jobj_nftables);
+	for(i = 0; i < arraylen; i++) {
+		json_object *jobj = json_object_array_get_idx(jobj_nftables, i);
+		if (jobj == NULL) {
+			debug(LOG_ERR, "fw4_counters_update(): jobj is NULL");
+			continue;
+		}
+		// get rule json object
+		json_object *jobj_rule = NULL;
+		if (!json_object_object_get_ex(jobj, "rule", &jobj_rule)) {
+			debug(LOG_ERR, "fw4_counters_update(): jobj_rule is NULL");
+			continue;
+		}
+		// get expr json object
+		// the expr json object is an array of json objects
+		json_object *jobj_expr = NULL;
+		if (!json_object_object_get_ex(jobj_rule, "expr", &jobj_expr)) {
+			debug(LOG_ERR, "fw4_counters_update(): jobj_expr is NULL");
+			continue;
+		}
+		// iterate the expr array to get each json object
+		// find the json object which has the "counter" key
+		// find the json object which has the "match" key
+		/*
+		 {
+            "match": {
+              "op": "==",
+              "left": {
+                "payload": {
+                  "protocol": "ip",
+                  "field": "saddr"
+                }
+              },
+              "right": "192.168.1.18"
+            }
+          },
+		*/
+		int j = 0;
+		int expr_arraylen = json_object_array_length(jobj_expr);
+		uint64_t packets = 0, bytes = 0;
+		t_client *p1 = NULL;
+		for (; j < expr_arraylen; j++) {
+			json_object *jobj_expr_item = json_object_array_get_idx(jobj_expr, j);
+			if (jobj_expr_item == NULL) {
+				debug(LOG_INFO, "fw4_counters_update(): jobj_expr_item is NULL");
+				continue;
+			}
+			json_object *jobj_counter = NULL;
+			json_object *jobj_match = NULL;
+			if (json_object_object_get_ex(jobj_expr_item, "counter", &jobj_counter)) {
+				// find the json object which has the "counter" key
+				// get packets and bytes
+				json_object *jobj_packets = NULL;
+				json_object *jobj_bytes = NULL;
+				if (!json_object_object_get_ex(jobj_counter, "packets", &jobj_packets)) {
+					debug(LOG_INFO, "fw4_counters_update(): jobj_packets is NULL");
+					continue;
+				}
+				if (!json_object_object_get_ex(jobj_counter, "bytes", &jobj_bytes)) {
+					debug(LOG_INFO, "fw4_counters_update(): jobj_bytes is NULL");
+					continue;
+				}
+				packets = json_object_get_int64(jobj_packets);
+				bytes = json_object_get_int64(jobj_bytes);
+			} else if (p1 == NULL && json_object_object_get_ex(jobj_expr_item, "match", &jobj_match)) {
+				json_object *jobj_match_right = NULL;
+				if (!json_object_object_get_ex(jobj_match, "right", &jobj_match_right)) {
+					debug(LOG_INFO, "fw4_counters_update(): jobj_match_right is NULL");
+					continue;
+				}
+				// get ip or mac address
+				const char *ip_or_mac = json_object_get_string(jobj_match_right);
+				if (ip_or_mac == NULL) {
+					debug(LOG_INFO, "fw4_counters_update(): ip_or_mac is NULL");
+					continue;
+				}
+				// find client by ip or mac address
+				// check whether the ip_or_mac is ip address or mac address
+				if (is_valid_ip(ip_or_mac)) {
+					p1 = client_list_find_by_ip(ip_or_mac);
+				} else if (is_valid_mac(ip_or_mac)) {
+					p1 = client_list_find_by_mac(ip_or_mac);
+				} else {
+					debug(LOG_INFO, "fw4_counters_update(): ip_or_mac is not ip address or mac address");
+				}
+			}
+		} 
+		if (p1) {
+			if (p1->name == NULL) {
+				debug(LOG_INFO, "fw4_counters_update(): p1->name is NULL");
+				__get_client_name(p1);
+			}
+
+			if(p1->wired == -1) {
+				p1->wired = br_is_device_wired(p1->mac);
+			}
+
+			p1->counters.outgoing_history = p1->counters.outgoing;
+			p1->counters.outgoing = bytes;
+			p1->counters.outgoing_packets = packets;
+			p1->counters.last_updated = time(NULL);
+			p1->is_online = 1;
+		} else {
+			debug(LOG_INFO, "fw4_counters_update(): p1 is NULL");
+		}
+	}
+
+	// get client outgoing traffic
+	char incoming[4096] = {0};
+	nft_statistical_incoming(incoming, sizeof(incoming));
+	// incoming is json format string
+	// parse json string incoming as jobj_incoming
+	json_object *jobj_incoming = json_tokener_parse(incoming);
+	if (jobj_incoming == NULL) {
+		debug(LOG_ERR, "fw4_counters_update(): jobj_incoming is NULL");
+		nret = 1;
+		goto END_UPDATE;
+	}
+	// get the "nftables" json object which is an array of json objects
+	if (!json_object_object_get_ex(jobj_incoming, "nftables", &jobj_nftables)) {
+		debug(LOG_ERR, "fw4_counters_update(): jobj_nftables is NULL");
+		nret = 1;
+		goto END_UPDATE;
+	}
+	// iterate the array of json objects in jobj_incoming
+	// get the json object which has the "rule" key
+	// get the json object which has the "expr" key
+	arraylen = json_object_array_length(jobj_nftables);
+	for(i = 0; i < arraylen; i++) {
+		json_object *jobj = json_object_array_get_idx(jobj_nftables, i);
+		if (jobj == NULL) {
+			debug(LOG_INFO, "fw4_counters_update(): jobj is NULL");
+			continue;
+		}
+		json_object *jobj_rule = NULL;
+		if (!json_object_object_get_ex(jobj, "rule", &jobj_rule)) {
+			debug(LOG_ERR, "fw4_counters_update(): jobj_rule is NULL");
+			continue;
+		}
+		// get expr json object
+		// the expr json object is an array of json objects
+		json_object *jobj_expr = NULL;
+		if (!json_object_object_get_ex(jobj_rule, "expr", &jobj_expr)) {
+			debug(LOG_ERR, "fw4_counters_update(): jobj_expr is NULL");
+			continue;
+		}
+		// iterate the array of json objects in jobj_expr
+		// get the json object which has the "counter" key
+		// get the json object which has the "match" key
+		// get the json object which has the "right" key
+		// get the ip address from the json object which has the "right" key
+		// find client by ip address
+		// get packets and bytes
+		int j = 0;
+		int expr_arraylen = json_object_array_length(jobj_expr);
+		uint64_t packets = 0;
+		uint64_t bytes = 0;
+		t_client *p1 = NULL;
+		for(; j < expr_arraylen; j++) {
+			json_object *jobj_expr_item = json_object_array_get_idx(jobj_expr, j);
+			if (jobj_expr_item == NULL) {
+				debug(LOG_INFO, "fw4_counters_update(): jobj_expr_item is NULL");
+				continue;
+			}
+			json_object *jobj_counter = NULL;
+			json_object *jobj_match = NULL;
+			if (json_object_object_get_ex(jobj_expr_item, "counter", &jobj_counter)) {
+				// find the json object which has the "counter" key
+				// get packets and bytes
+				json_object *jobj_packets = NULL;
+				json_object *jobj_bytes = NULL;
+				if (!json_object_object_get_ex(jobj_counter, "packets", &jobj_packets)) {
+					debug(LOG_INFO, "fw4_counters_update(): jobj_packets is NULL");
+					continue;
+				}
+				if (!json_object_object_get_ex(jobj_counter, "bytes", &jobj_bytes)) {
+					debug(LOG_INFO, "fw4_counters_update(): jobj_bytes is NULL");
+					continue;
+				}
+				packets = json_object_get_int64(jobj_packets);
+				bytes = json_object_get_int64(jobj_bytes);
+			} else if (p1 == NULL && json_object_object_get_ex(jobj_expr_item, "match", &jobj_match)) {
+				json_object *jobj_match_right = NULL;
+				if (!json_object_object_get_ex(jobj_match, "right", &jobj_match_right)) {
+					debug(LOG_INFO, "fw4_counters_update(): jobj_match_right is NULL");
+					continue;
+				}
+				// get ip address from the json object which has the "right" key
+				const char *ip = json_object_get_string(jobj_match_right);
+				if (ip == NULL || is_valid_ip(ip) == 0) {
+					debug(LOG_INFO, "fw4_counters_update(): ip is NULL");
+					continue;
+				}
+				// find client by ip address
+				p1 = client_list_find_by_ip(ip);
+			}
+		}
+		if (p1) {
+			p1->counters.incoming_history = p1->counters.incoming;
+			p1->counters.incoming = bytes;
+			p1->counters.incoming_packets = packets;
+		} else {
+			debug(LOG_INFO, "fw4_counters_update(): p1 is NULL");
+		}
+	}
+
+END_UPDATE:
+	// free jobj_outgoing and jobj_incoming
+	if (jobj_outgoing) json_object_put(jobj_outgoing);
+	if (jobj_incoming) json_object_put(jobj_incoming);
+	return nret;
+}
+
+#endif
+
+/** Update the counters of all the clients in the client list */
+int
+iptables_fw_counters_update(void)
+{
+#ifdef AW_FW3
+	return fw3_counters_update();
+#else
+	return fw4_counters_update();
+#endif
+}
\ No newline at end of file
