@@ -79,6 +79,7 @@ static void wdctl_del_trusted_local_maclist(struct bufferevent *, const char *);
 static void wdctl_add_untrusted_maclist(struct bufferevent *, const char *);
 static void wdctl_del_untrusted_maclist(struct bufferevent *, const char *);
 static void wdctl_add_online_client(struct bufferevent *, const char *);
+static void wdctl_add_auth_client(struct bufferevent *, const char *);
 
 static struct wd_request_context *request_ctx;
 
@@ -120,7 +121,8 @@ static struct wdctl_command {
     {"del_trusted_local_mac", NULL, wdctl_del_trusted_local_maclist},
     {"add_untrusted_mac", NULL, wdctl_add_untrusted_maclist},
     {"del_untrusted_mac", NULL, wdctl_del_untrusted_maclist},
-    {"add_online_client", NULL, wdctl_add_online_client}
+    {"add_online_client", NULL, wdctl_add_online_client},
+	{"add_auth_client", NULL, wdctl_add_auth_client},
 };
 
 static void 
@@ -197,9 +199,10 @@ wdctl_listen_new_client(struct evconnlistener *listener, evutil_socket_t fd,
     bufferevent_enable(b_client, EV_READ|EV_WRITE);
 }
 
-/** Launches a thread that monitors the control socket for request
-@param arg Must contain a pointer to a string containing the Unix domain socket to open
-*/
+/** 
+ * Launches a thread that monitors the control socket for request
+ *  @param arg Must contain a pointer to a string containing the Unix domain socket to open
+ */
 void
 thread_wdctl(void *arg)
 {
@@ -783,3 +786,38 @@ OUT:
     bufferevent_write(fd, "Yes", 3);
 }
 
+/**
+ *@brief add auth client for testing
+ *
+ */
+static void
+wdctl_add_auth_client(struct bufferevent *fd, const char *args)
+{
+	json_object *client_info = json_tokener_parse(args);
+	if(is_error(client_info) || json_object_get_type(client_info) != json_type_object) { 
+        goto OUT;
+    }
+
+    json_object *mac_jo = NULL;
+    json_object *ip_jo 	= NULL;
+    json_object *name_jo = NULL;
+    if(!json_object_object_get_ex(client_info, "mac", &mac_jo) || 
+	   !json_object_object_get_ex(client_info, "ip", &ip_jo) || 
+	   !json_object_object_get_ex(client_info, "name", &name_jo)) { 
+        goto OUT;
+    }
+
+    const char *mac  = json_object_get_string(mac_jo);
+    const char *ip	 = json_object_get_string(ip_jo);
+    if (!is_valid_mac(mac) || !is_valid_ip(ip) || !is_trusted_mac(mac))
+        goto OUT;
+
+    auth_req_info *auth = safe_malloc(sizeof(auth_req_info));
+    memcpy(auth->ip, ip, strlen(ip));
+    memcpy(auth->mac, mac, strlen(mac));
+    make_auth_request(request_ctx, auth);
+    
+OUT:
+    if (client_info) json_object_put(client_info);
+    bufferevent_write(fd, "Yes", 3);
+}
